#!/bin/bash
# 2010 niekt0@hysteria.sk 
# Under GNU/GPL

# This is just simple script for listing "all" processes.
# Take it as example, or modify to satisfy your needs

#XXX utility paths
foriana=./foriana
bgrep=./bgrep

if [ -z "$1" ]; then
	echo "Usage: $0 dump_file [options to bgrep]";
	echo "Cache file must be available already."
	echo "Example: ./$0 freebsd_256M_i386.dd \"-e 10 -s 1\""
	exit;
fi

if [ ! -f "$1" ]; then
	echo "Dump file does not exists."
	exit;
fi

# $1 = dump file
file=`basename "$1"`;
errcount="$2";

if [ ! -f ".$file" ]; then
	echo "Cache file \".$file\" not found";
	exit;
fi


if [ ! -x "$foriana" ]; then
	echo "executable $foriana not found.";
	exit;
fi

if [ ! -x "$bgrep" ]; then
	echo "executable $bgrep not found.";
	exit;
fi


# get process pattern
echo -n "Generating process pattern: "
$foriana --create-process-pattern --list-skip-areas "$1" | grep -A 1 "PROCESS PATTERN START" | tail -1 > ."$file"_process_pattern && echo "OK, saved to \"."$file"_process_pattern\"";


# Sometimes helps to ignore NZ bytes, and use then as ??
# cat "$file"_process_pattern | sed 's%/NZ/??/g' > "$file"_process_pattern_...

# optimize pattern => remove leading ?? and remember their number
# matching ?? at beginning of pattern is CPU expensive
cat ."$file"_process_pattern | sed 's/^?*//' > ."$file"_process_pattern_optimized
len1=`wc -c ."$file"_process_pattern | cut -d ' ' -f 1`;
len2=`wc -c ."$file"_process_pattern_optimized | cut -d ' ' -f 1`;
opt_offset=$((($len1 - $len2) / 2));

# find all occurrences of pattern
echo -n "Finding all occurrences: "
$bgrep $2 `cat ."$file"_process_pattern_optimized` "$1" > ."$file"_processes && echo "OK, saved to \"."$file"_processes\"";
processes_count=`wc -l ."$file"_processes | cut -d ' ' -f 1`;

# to see names instantly
# cache file will be changed to XML format
proc_offset=`cat ."$file" | grep '^ts_hl_comm_dst:' | cut -d : -f 2`;
echo "Using offset: $proc_offset";

echo "Structure addr: String addr: xxd format (chars after 0x00 can reveal fork history)";
# print addresses + process names
cat ."$file"_processes | cut -d ' ' -f 2 | while read i; do i=$((0x$i-$opt_offset)) && printf "%llx: " "$i" && j=$(($i + $proc_offset)) && xxd -l 16 -s "$j" "$1"; done
echo "Found $processes_count processes.";

