#ifndef _X86_64_FUNCTIONS_H
#define _X86_64_FUNCTIONS_H

/*
 *	Functions for analysing x86_64 architecture systems
*/

#include "../../globals.h"

#include "magic.h" 


/*
 * Initialization function.
 */

int x86_64_init(struct arch_struct *);



/*
 * Function that sets pointers to all other 
 * architecture dependent functions and hooks
 *
 */

int x86_64_arch_settings();


/*
 * Function that manages mapping from linear to physical address space.
 * 
 */

int x86_64_read_linear(u_int64_t PGD, u_int64_t lin_addr,
			void *out_data, u_int64_t priv_block_size,
			u_int64_t *dump_offset );


/*
 * Function for verifying if given address contains VPT 
 * (resp. PaGeDirectory) of some process.
 */
int x86_64_user_VPT_test(u_int64_t my_addr);

/*
 * Function for verifying if given address contains system VPT 
 * (resp. PaGeDirectory)
 *
 */
int x86_64_VPT_test(u_int64_t my_addr);

/*
 * Function that tries to determine general information 
 * about analyzed image.
 *
 * for now, only system VPT is found.
 * if mode == 1, then all possible VPT candidates are printed.
 *
 * x86_64_find_VPT ()
 * return 0 on success, otherwise error > 0
 * Discovered information are saved in global structure image1.
 *
 */
int x86_64_find_VPT(int mode);

#endif
