/// Global defines and includes.
/**
 * @file
 */

#ifndef _GLOBALS_H
#define _GLOBALS_H

#define fversion "0.9.0" //Change on release

// This is needed for opening files bigger than 2GB
#define _FILE_OFFSET_BITS 64

/*
 * System libraries 
 */

#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <sys/timeb.h>
#include <errno.h>
#include <libgen.h>

#include <math.h> // for round makro
#include "debug.h" // for dbgprint

#define round(x) ((x)>=0?(int)((x)+0.5):(int)((x)-0.5))
#define min(x,y) (x<y?x:y) //
#define max(x,y) (x>y?x:y) //


// Bits up to xth bit (64 bit version)
#define FULLBIT64(x) ((x)==-1)?((u_int64_t)0): \
	((u_int64_t)0xFFFFFFFFFFFFFFFFLLU >> (63-(x)))

// Bits up to xth bit (32 bit version)
#define FULLBIT32(x) ((x)==-1)?((u_int32_t)0): \
	((u_int32_t)0xFFFFFFFFLU >> (31-(x)))

// Create constant with bits from x to y set (64 bit version)
// for example B64(21,29) -> 0x000000003FE00000LLU
#define B64(x,y) ((u_int64_t)(FULLBIT64(y)) -\
		(u_int64_t)(FULLBIT64((x)-1))) 

// Create constant with bits from x to y set (32 bit version)
// for example B32(12,17) -> 0x3F000LU
#define B32(x,y) ((u_int32_t)(FULLBIT32(y)) - \
		(u_int32_t)(FULLBIT32((x)-1))) 

// bits from x to y, extracted & shifted (64 bit version)
// for example BS64(0xFFFFFFFFFFFFFFFF,21,29) -> 
//((var & B64(21,29))>> 21) = 0x00000000000001FFLLU
#define BS64(var,x,y) ((((var) & (B64((x),(y)))) >> (x) )) 

// bits from x to y, extracted & shifted (32 bit version)
// for example BS32(0xFFFFFFFF,21,29) -> 
//((var & B32(21,29))>> 21) = 0x000001FFLLU
#define BS32(var,x,y) ((((var) & (B32((x),(y)))) >> (x) )) 



#define _GNU_SOURCE		// ;)
//how many lines (variables) are stored in "cache" files
#define FORIANA_CACHE_SIZE 13
#define LINE_SIZE 80		// size of terminal

#include "helper.h"		// various help rutines
#include "structs.h" 		// structure definitions
#include "arch/allarch.h"	// needed for global variables
#include "os/allos.h"		// needed for global variables

/*
 * Forward declarations:
 */

int list_head_size(u_int64_t listhead);
int search_init(char *inputfile2,const char *marker2,
		u_int64_t start_addr);
u_int64_t search_next();
int search_exit();


/*
 * Global variables
 */

extern struct my_image img1;
extern struct arch_struct architectures[MAX_ARCHITECTURES];
extern struct os_struct systems[MAX_OS];

extern int dbg;

//char helpstatusfile[20]="foriana.cache";
extern char *statusfile2,*statusfile;
extern FILE *fs;	//cache file

extern char *inputfile;	//filename
extern int inputfd,statusfd; //input & status output 

extern struct stat sbuffer;

extern char *errormsg;



extern int status;
//extern unsigned long pointer;

//global states of parsed options
extern int do_files_det,do_files_list,do_areas_list,ddebug; 
//global states of parsed options
extern int do_create_proc_pattern; 
//global states of parsed options
extern int do_create_module_pattern; 

extern int magic_module_set,magic_process_set,magic_user_process_set;



#endif
